%% ex_TMsim_RF_Filter.m
%
% Reproduces the TM and MC results of the example "RF filter" (see Fig. 8
% and 9) in [1] Sec. VI
%
% [1] R. Trinchero, P. Manfredi, T. Ding and I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models," in IEEE
% Tran. on Circuits and Systems I: Regular Papers, vol. 63, no. 7,
% pp. 1067-1078, July 2016.
%
% This example script is part of the tool TMsim (TMsim.zip)
% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


% cleaning
close all;
clear all;
clc

% Frequency data
Nf = 30;
freq = linspace(0.1,1e8,Nf);

w = 2*pi*freq;

% MC runs
Nruns = 10000; 

% Circuit parameters
G = 1/1;

% Components
L1n = 2.92e-9;
C1n = 3.68e-9;
L2n = 6.25e-9;
C2n = 5.83e-9;
L3n = 1.84e-9;
C3n = 12.5e-9;

% Variation & MC Values
delta = 0.05;

C1d = C1n*0;
C2d = C2n*0;
C3d = C3n*0;

L1d = L1n*delta;
L2d = L2n*delta;
L3d = L3n*delta;

%% ========================================================================
% Monte Carlo (uncorrelated variation, 3 independent parameters)
% =========================================================================
tic

x1 = 2*rand(Nruns,1)-1;
x2 = 2*rand(Nruns,1)-1;
x3 = 2*rand(Nruns,1)-1;
x4 = 2*rand(Nruns,1)-1;
x5 = 2*rand(Nruns,1)-1;
x6 = 2*rand(Nruns,1)-1;

E = 1;

H= zeros(Nf,Nruns);

dim_MNA = 10;
MNA = zeros(dim_MNA);

progressbar(0);

for J = 1:Nf
    for K=1: Nruns        
            
            C1 = C1n+C1d*x1(K);
            C2 = C2n+C2d*x2(K);
            C3 = C3n+C3d*x3(K);
            
            L1 = L1n+L1d*x4(K);
            L2 = L2n+L2d*x5(K);
            L3 = L3n+L3d*x6(K);
            
            %Circuit excitation
            A = zeros(10,1);
            A(1)= E*G;
            
            MNA(1,1) = G+1j*w(J)*(C1+C2);
            MNA(1,2) = -1j*w(J)*C1;
            MNA(1,3) = -1j*w(J)*C2;
            MNA(1,6) = 1;
            
            MNA(2,1) = -1j*w(J)*C1;
            MNA(2,2) = 1j*w(J)*C1;
            MNA(2,7) = 1;
            
            MNA(3,1) = -1j*w(J)*C2;
            MNA(3,3) = 1j*w(J)*(C2+C3);
            MNA(3,4) = -1j*w(J)*C3;
            MNA(3,8) = 1;
            
            MNA(4,3) = -1j*w(J)*C3;
            MNA(4,4) = G+1j*w(J)*(C3+C1);
            MNA(4,5) = -1j*w(J)*C1;
            MNA(4,8) = -1;
            MNA(4,9) = 1;
            
            MNA(5,4) = -1j*w(J)*C1;
            MNA(5,5) = 1j*w(J)*C1;
            MNA(5,10) = 1;
            
            MNA(6,1) = 1;
            MNA(6,6) = -1j*w(J)*L1;

            MNA(7,2) = 1;
            MNA(7,7) = -1j*w(J)*L2;
            
            MNA(8,3) = 1;
            MNA(8,4) = -1;
            MNA(8,8) = -1j*w(J)*L3;
            
            MNA(9,4) = 1;
            MNA(9,9) = -1j*w(J)*L1;
            
            
            MNA(10,5) = 1;
            MNA(10,10) = -1j*w(J)*L2;                                                       
            
            tmp = inv(MNA)*A; 
            
            H(J,K) = 2*tmp(4);            
    end
    progressbar(J/Nf);
end
    
MC_time=toc

% real part of H
figure(1)
hPtmp = plot(freq,real(H),'k-','color',[0.8,0.8,0.8]); hold on;
title('Real Part 1')
hP_r(1) = hPtmp(1);

% imag part of H
figure(2)
hPtmp = plot(freq,imag(H),'k-','color',[0.8,0.8,0.8]); hold on;
title('Imag Part 1')
hP_i(1) = hPtmp(1);

% dB part of H
figure(3)
hPtmp = plot(freq,(abs(H)),'k-','color',[0.8,0.8,0.8]); hold on;
title('ABS Part')
hP_i(1) = hPtmp(1);



%% ========================================================================
% Taylor model (order 3)
% =========================================================================
tic;

N_var = 3; % Number of variables
N_order = 3; % Expansion order

BN_Base_extra = BN_Basis(2*N_order,-1,1);

% inizializing TMsim
initialize_TMsim(N_order,N_var);

% defining the TM variables
CT1 = TaylorModel(C1n,C1d,1);
CT2 = TaylorModel(C2n,C2d,1);
CT3 = TaylorModel(C3n,C3d,1);

LT1 = TaylorModel(L1n,L1d,1);
LT2 = TaylorModel(L2n,L2d,2);
LT3 = TaylorModel(L3n,L3d,3);

% Taylor model of 1
OneT = TaylorModel(1,0,1);

% Taylor model of -1
OnemT = TaylorModel(-1,0,1);

% Taylor model of 0
ZeroT = TaylorModel(0,0,1);

% Empty MNA matrix
dim_MNA = 10;

for K=1:dim_MNA
    for I=1:dim_MNA
        MNA_TM(K,I) = ZeroT;
    end
end

BMag_IA = zeros(Nf,2);
BMag_TM = zeros(Nf,2);

%Circuit excitation
A = zeros(10,1);
A(1)= E*G;

progressbar(0);

% MNA and TMs
MNA_TM(1,6) = OneT;
MNA_TM(2,7) = OneT;
MNA_TM(3,8) = OneT;
MNA_TM(4,8) = OnemT;
MNA_TM(4,9) = OneT;
MNA_TM(5,10) = OneT;
MNA_TM(6,1) = OneT;
MNA_TM(7,2) = OneT;
MNA_TM(8,3) = OneT;
MNA_TM(8,4) = OnemT;
MNA_TM(9,4) = OneT;
MNA_TM(10,5) = OneT;


for J = 1:Nf        

    MNA_TM(1,1) = G+1j*w(J)*(CT1+CT2);
    MNA_TM(1,2) = -1j*w(J)*CT1;
    MNA_TM(1,3) = -1j*w(J)*CT2;

    MNA_TM(2,1) = -1j*w(J)*CT1;
    MNA_TM(2,2) = 1j*w(J)*CT1;

    MNA_TM(3,1) = -1j*w(J)*CT2;
    MNA_TM(3,3) = 1j*w(J)*(CT2+CT3);
    MNA_TM(3,4) = -1j*w(J)*CT3;

    MNA_TM(4,3) = -1j*w(J)*CT3;
    MNA_TM(4,4) = G+1j*w(J)*(CT3+CT1);
    MNA_TM(4,5) = -1j*w(J)*CT1;

    MNA_TM(5,4) = -1j*w(J)*CT1;
    MNA_TM(5,5) = 1j*w(J)*CT1;

    MNA_TM(6,6) = -1j*w(J)*LT1;

    MNA_TM(7,7) = -1j*w(J)*LT2;


    MNA_TM(8,8) = -1j*w(J)*LT3;

    MNA_TM(9,9) = -1j*w(J)*LT1;

    MNA_TM(10,10) = -1j*w(J)*LT2;                                                       

    % inverting the MNA matrix
    invMNA_TM = Sherman_inv_matrix(MNA_TM);

    % solving the circuit
    tmp = invMNA_TM*A;

    HT = 2*tmp(4);

    %max and min of both the real and imag part

    HT_r.min(J) = real(HT.B(1) + HT.I(1));
    HT_i.min(J) = imag(HT.B(1) + HT.I(1));
    HT_r.max(J) = real(HT.B(2) + HT.I(2));
    HT_i.max(J) = imag(HT.B(2) + HT.I(2));

    BMag = TM_abs_bound(HT,'lin');

    % Physical constraint
    %the magnitude of the scattering parameters <1
    BMag_best(J,1) = BMag(1);
    BMag_best(J,2) = min(1,BMag(2));


    progressbar(J/Nf);
end

TM_time=toc

figure(1)
hP_r(2) = semilogx(freq,HT_r.min,'m--','linewidth',1.5); hold on;
semilogx(freq,HT_r.max,'m--','linewidth',1.5); hold on;
set(gca,'xlim',[freq(1),freq(end)]);
xlim([min(freq),max(freq)])
legend(hP_r,'MC (indep.)','TM','location','southwest');

figure(2)
hP_i(2) = semilogx(freq,HT_i.min,'m--','linewidth',1.5); hold on;
semilogx(freq,HT_i.max,'m--','linewidth',1.5); hold on;
set(gca,'xlim',[freq(1),freq(end)]);
xlim([min(freq),max(freq)])
legend(hP_i,'MC (indep.)','TM','location','southwest');

figure(3)
semilogx(freq,BMag_best,'b-','linewidth',2); hold on;

