%% ex_TMsim_coaxial_1var.m
%
% Reproduces the TM and MC results of application example #1 "Coaxial
% Cable" (see Fig. 2) in [4] Sec. III.A
%
% [4] P. Manfredi, R. Trinchero, I. S. Stievano "Worst-Case EMC
% Investigation of Single-Wire Transmission Lines Based on Taylor
% Arithmetic," submitted to EMC Symphosium 2017.
% 
% This example script is part of the tool TMsim (TMsim.zip)
% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

% cleaning
clear all
clc
close all

% plot parameters
FS=14;
FN='Times';
FW='normal';
LW=1;

GRAY = [0.8,0.8,0.8];

% physical constants
mu0 = 4*pi*1e-7;
eps0 = 8.854e-12;
c0 = 1/sqrt(mu0*eps0);

% line data
rw = 0.4064e-3;
rsh = 1.4732e-3;
er = 2.3;
len = 1;

% line terminations
lin.Vs = 1;
lin.Zs = 50;

lin.Il = 0;
Cl = 1e-12;

% variability
var = 0.25;

% frequency sweep data
Nf = 401;
freq = logspace(7,9,Nf);

% Monte Carlo runs
Nruns = 10000;
yruns = 2*rand(Nruns,1)-1;

% =========================================================================
% Monte Carlo simulation
% =========================================================================

V_0 = zeros(Nf,Nruns);
V_L = zeros(Nf,Nruns);

tic
progressbar(0)
for J = 1:Nruns
    rshnew = rsh*(1+var*yruns(J,1));
    
    lin.L = 2e-7 * log(rshnew/rw);
    lin.C = 2*pi*er*eps0 / log(rshnew/rw);
    
    for I = 1:Nf
        w = 2*pi*freq(I);
        beta = sqrt(er)*w/c0;
        
        lin.Yl = 1j*w*Cl;
        
        PHI_11 = cos(beta*len);
        PHI_12 = -1j*c0/sqrt(er)*sin(beta*len)*lin.L;
        PHI_21 = -1j*c0/sqrt(er)*sin(beta*len)*lin.C;
        PHI_22 = cos(beta*len);
        
        % Current at near-end side of the line
        A = PHI_21*lin.Zs + lin.Yl*PHI_12 - PHI_22 - lin.Yl*PHI_11*lin.Zs;
        B = (PHI_21 - lin.Yl*PHI_11)*lin.Vs;
        
        I_0 = B/A;
        
        % Voltage at near-end side of the line
        V_0(I,J) = lin.Vs - lin.Zs*I_0;
        
        % Voltage at far.end side of the line
        V_L(I,J) = PHI_11*lin.Vs + (PHI_12 - PHI_11*lin.Zs)*I_0;
              
    end
    
    progressbar(J/Nruns);
end
toc

% MC bounds
UB_V0 = max(20*log10(abs(V_0)),[],2);
LB_V0 = min(20*log10(abs(V_0)),[],2);
UB_VL = max(20*log10(abs(V_L)),[],2);
LB_VL = min(20*log10(abs(V_L)),[],2);

figure(1)
hA(1) = area(freq,UB_VL,'facecolor',GRAY,'basevalue',-2.4,'showbaseline','off'); hold on; % upper bound
hA(2) = area(freq,LB_VL,'facecolor',[1,1,1],'basevalue',-2.4,'showbaseline','off'); hold on; % lower bound
set(gca,'xscale','log');
set(hA,'edgecolor','none');
set(gcf,'color',[1,1,1]);
set(gcf, 'InvertHardCopy', 'off');

hP(1) = hA(1);

% =========================================================================
% Taylor model simulation
% =========================================================================

N_order = 8;
initialize_TMsim(N_order,1);

% Taylor model of R1 
sqrt_er_T = TaylorModel(sqrt(er),0,1);

% Taylor model of 1
OneT = TaylorModel(1,0,1);

% Taylor model of -1
OnemT = TaylorModel(-1,0,1);

% Taylor model of 0
ZeroT = TaylorModel(0,0,1);

% Taylor model of rsh
rsh_T = TaylorModel(rsh*(1-var),rsh*(1+var),1,'bound');

% Taylor model of L
L_T = 2e-7 * log((1/rw)*rsh_T);

% Taylor model of C0
C0_T = 2*pi*eps0*TM_inv_scalar(log((1/rw)*rsh_T));
    
vTf = c0*TM_inv_scalar(sqrt_er_T);
ZTc = vTf*L_T;
YTc = sqrt_er_T*c0*C0_T;
   
tic
progressbar(0)
for I = 1:Nf
    w = 2*pi*freq(I);
    betaT = sqrt_er_T*(w/c0);

    YTL = 1j*w*Cl;

    PHI_11 = TM_cos(betaT*len);
    PHI_12 = -1j*ZTc*TM_sin(betaT*len);
    PHI_21 = -1j*YTc*TM_sin(betaT*len);
    PHI_22 = TM_cos(betaT*len);
    
   % Current at near-end side of the line
    A = PHI_21*lin.Zs + YTL*PHI_12 - PHI_22 - YTL*PHI_11*lin.Zs;
    B = (PHI_21 - YTL*PHI_11)*lin.Vs;
    
    IT_0 = TM_inv_scalar(A)*B;
    
    % Voltage at near-end side of the line
    VT_0 = lin.Vs - lin.Zs*IT_0;

    % Voltage at far.end side of the line
    VT_L = PHI_11*lin.Vs + (PHI_12 - PHI_11*lin.Zs)*IT_0;

    % Current at far-end side of the line
    IT_L = YTL*VT_L;

    T_Mag_V0(I,:) = TM_abs_bound(VT_0);
    T_Mag_VL(I,:) = TM_abs_bound(VT_L);

    progressbar(I/Nf);
end
toc

figure(1)
htmp = semilogx(freq,T_Mag_VL,'b','linewidth',2); hold on;
hP(2) = htmp(1);
set(gca,'FontName',FN,'FontSize',FS,'Linewidth',LW,'FontWeight',FW);
hL = legend(gca,hP,'MC','TA bounds','location','southwest');
set(gca,'ylim',[-2.5,2.0]);
xlabel('Frequency [Hz]','interpreter','latex');
ylabel('Far-end voltage [dBV]','interpreter','latex');
set(gca,'position',[0.2,0.2,0.7,0.55]);
