%% ex_TMsim_plane_wave.m
%
% Reproduces the TM and MC results of application example #2 "Plane Wave
% Incidence on a Wire Above Ground" (see Fig. 3) in [4] Sec. III.B
%
% [4] P. Manfredi, R. Trinchero, I. S. Stievano "Worst-Case EMC
% Investigation of Single-Wire Transmission Lines Based on Taylor
% Arithmetic," submitted to EMC Symphosium 2017.
% 
% This example script is part of the tool TMsim (TMsim.zip)
% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reproduces by means of MC method example # 1 (see Fig. 5) in [1] Sec. IV
%
% [1] D. Bellan, S. Pignari, "A Probabilistic Model for the Response of an
% Electrically Short Two-Conductor Transmission Line Driven by a Random
% Plane Wave Field", IEEE Transactionf on Electromagnetic Compatibility
%
% Random FIELD and ELEVATION

% Cleaning
close all
clear all
clc

% plot parameters
FS=14;
FN='Times';
FW='normal';
LW=1;
GRAY = [0.8,0.8,0.8];

%% Summary:
%
% Structure: wire above a ground plane
% rw = 0.5 mm
% h = 10 cm
% length = 1 m
%
% loads = Zc

%% Parameters
% Constants
mu0 = 4*pi*1e-7;
eps0 = 8.854187817e-12;
c0 = 1/sqrt(eps0*mu0);

% Line parameters
rw = 0.5e-3; % wire radius
h = 0.1; % wire height above ground
len = 1; % wire length

% Wave parameters
wave.E0 = 1;
wave.thp = pi/6;
wave.phip = pi/4; % NB phip != psi (phip = psi - pi/2)

% Simulation parameters
Nf = 401;
freq = logspace(7,9,Nf);

Nruns = 10000; % runs for PDF computation

% Line parameters
lin.L = 2e-7*acosh(h/rw); % per-unit-length inductance
lin.C = 2*pi*eps0/acosh(h/rw); % per-unit-length capacitance
Zc = sqrt(lin.L/lin.C);
lin.Vs = 0; % terminations
lin.Vl = 0;
lin.Zs = Zc;
lin.Zl = Zc;
lin.len = len;
lin.pos = [0 h]; % position
lin.R = 0;
lin.G = 0;

%% Monte Carlo

% Samples
yruns = 2*rand(Nruns,1)-1;

VL_MC = zeros(Nf,Nruns);

% Initialization: TL parameters do not depend on impinging field, just on
% frequency => they are evaluated once
thE_av = pi/2; % range of variation can be actually halved due to symmetry
thE_dev = pi/2;

tic;
progressbar(0);
for J = 1:Nruns
    wave.thE = thE_av+thE_dev * yruns(J,1);
    
    % Field components
    ex = sin(wave.thE)*sin(wave.thp);
    ey = -sin(wave.thE)*cos(wave.thp)*cos(wave.phip) - cos(wave.thE)*sin(wave.phip);
    ez = -sin(wave.thE)*cos(wave.thp)*sin(wave.phip) + cos(wave.thE)*cos(wave.phip);
    
    Zc = c0*lin.L;
	Yc = c0*lin.C;
    
    for I = 1:Nf
    
        w = 2*pi*freq(I);
    
        b = w/c0;
        Gamma = 1j*b;
        bx = -b*cos(wave.thp);
        by = -b*sin(wave.thp)*cos(wave.phip);
        bz = -b*sin(wave.thp)*sin(wave.phip);

        % Field contribution
        Mp = -2j*wave.E0*ez*(exp(Gamma*len)*(1-exp(-(Gamma+1j*bz)*len))/(Gamma+1j*bz) + ...
            exp(-Gamma*len)*(exp((Gamma-1j*bz)*len)-1)/(Gamma-1j*bz))*(1j*w*lin.C*sin(bx*h));

        Mm = -2j*wave.E0*ez*(exp(Gamma*len)*(1-exp(-(Gamma+1j*bz)*len))/(Gamma+1j*bz) - ...
            exp(-Gamma*len)*(exp((Gamma-1j*bz)*len)-1)/(Gamma-1j*bz))*(1j*w*lin.C.*sin(bx*h));

        Np = 2*wave.E0*(exp(Gamma*len) + exp(-Gamma*len))*(ex*h*sin(bx*h)/(bx*h)*exp(-1j*bx*h))*(1j*w*lin.C);

        Nm = 2*wave.E0*(exp(Gamma*len) - exp(-Gamma*len))*(ex*h*sin(bx*h)/(bx*h)*exp(-1j*bx*h))*(1j*w*lin.C);

        ET = 2*wave.E0*ex*h*sin(bx*h)/(bx*h)*exp(-1j*bx*h)*exp(-1j*bz*len);

        % Forcing terms        
        V_FT = (Mp+Np)/(2j*w*lin.C) - ET;
        I_FT = -(Mm+Nm)/(2*Gamma);
        
        Phi_11 = cos(b*lin.len);
        Phi_12 = -1j*Zc*sin(b*lin.len);
        Phi_21 = -1j*Yc*sin(b*lin.len);
        Phi_22 = cos(b*lin.len);
        
        % Current at near-end side of the line
        A = Phi_11*lin.Zs + lin.Zl*Phi_22 -Phi_12 - lin.Zl*Phi_21*lin.Zs;
        B = (V_FT - lin.Zl*I_FT);
        I_0 = B/A;
        
        % Current at far-end side of the line
        I_L = I_FT + (Phi_22-Phi_21*lin.Zs)*I_0;
        
        % Voltage at near-end side of the line
        V_0 = - lin.Zs*I_0;
        
        % Voltage at far-end side of the line
        VL_MC(I,J) = lin.Zl*I_L;
    end
    
    progressbar(J/Nruns);
end

MC_time = toc

% MC bounds
UB_MC = max(20*log10(abs(VL_MC)),[],2);
LB_MC = min(20*log10(abs(VL_MC)),[],2);

figure(1)
hA(1) = area(freq,UB_MC,'facecolor',GRAY,'basevalue',-80.5,'showbaseline','off'); hold on; % upper bound
hA(2) = area(freq,LB_MC,'facecolor',[1,1,1],'basevalue',-80.5,'showbaseline','off'); hold on; % lower bound
set(gca,'xscale','log');
set(hA,'edgecolor','none');
set(gcf,'color',[1,1,1]);
set(gcf, 'InvertHardCopy', 'off');

hP(1) = hA(1);

%% FULL FREQUENCY DOMAIN SIMULATION - TAYLOR MODELS
VL_TM = zeros(Nf,2);

% TM parameter setup
N_order = 25;

initialize_TMsim(N_order,1);

% Initialization: TL parameters do not depend on impinging field, just on
% frequency => they are evaluated once
TthE = TaylorModel(thE_av,thE_dev,1);

TE0 = wave.E0;

Tex = sin(TthE)*sin(wave.thp);
Tey = -1*sin(TthE)*cos(wave.thp)*cos(wave.phip) -1* cos(TthE)*sin(wave.phip);
Tez = -1*sin(TthE)*cos(wave.thp)*sin(wave.phip) + cos(TthE)*cos(wave.phip);
    
Zc = c0*lin.L;
Yc = c0*lin.C;
    
    
tic
progressbar(0)
for I = 1:Nf
    
    w = 2*pi*freq(I);
    
    b = w/c0;
    Gamma = 1j*b;
    bx = -b*cos(wave.thp);
    %by = -b*sin(wave.thp)*cos(wave.phip);
    bz = -b*sin(wave.thp)*sin(wave.phip);
    
    % Field contribution
    TMp = -2j*TE0*Tez*(exp(Gamma*len)*(1-1*exp(-1*(Gamma+1j*bz)*len))*inv(Gamma+1j*bz) + ...
        exp(-Gamma*len)*(exp((Gamma-1j*bz)*len)-1)*inv(Gamma-1j*bz))*(1j*w*lin.C*sin(bx*h));
    
    TMm = -2j*TE0*Tez*(exp(Gamma*len)*(1-1*exp(-1*(Gamma+1j*bz)*len))*inv(Gamma+1j*bz) - ...
        exp(-Gamma*len)*(exp((Gamma-1j*bz)*len)-1)*inv(Gamma-1j*bz))*(1j*w*lin.C*sin(bx*h));
    
    TNp = 2*TE0*(exp(Gamma*len) + exp(-Gamma*len))*(Tex*h*sin(bx*h)*inv(bx*h)*exp(-1j*bx*h))*(1j*w*lin.C);
    
    TNm = 2*TE0*(exp(Gamma*len)-1*exp(-Gamma*len))*(Tex*h*sin(bx*h)*inv(bx*h)*exp(-1j*bx*h))*(1j*w*lin.C);
    
    TET = 2*TE0*Tex*h*sin(bx*h)*inv(bx*h)*exp(-1j*bx*h)*exp(-1j*bz*len);
    
    % Forcing terms    
    TV_FT = (TMp+TNp)*(1/(2j*w*lin.C)) - TET;
    TI_FT = -1*(TMm+TNm)*(1/(2*Gamma));
    
    Phi_11 = cos(b*lin.len);
    Phi_12 = -1j*Zc*sin(b*lin.len);
    Phi_21 = -1j*Yc*sin(b*lin.len);
    Phi_22 = cos(b*lin.len);
    
    % Current at near-end side of the line
    A = Phi_11*lin.Zs + lin.Zl*Phi_22 -Phi_12 - lin.Zl*Phi_21*lin.Zs;
    TB = (TV_FT - lin.Zl*TI_FT);
    TI_0 = inv(A)*TB;
    
    % Current at far-end side of the line
    TI_L = TI_FT + (Phi_22-Phi_21*lin.Zs)*TI_0;
    
    % Voltage at near-end side of the line
    TV_0 = - lin.Zs*TI_0;
    
    % Voltage at far-end side of the line
    TV_L = lin.Zl*TI_L;
    
    TVL(I) = TV_L;
    
    VL_abs_TM(I,:) = TM_abs_bound(TVL(I));
    
    progressbar(I/Nf);
end

TM_time = toc

% plot
figure(1)
htmp = semilogx(freq,(VL_abs_TM(:,2)),'b-','linewidth',2); hold on;
hP(2) = htmp(1);
set(gca,'FontName',FN,'FontSize',FS,'Linewidth',LW,'FontWeight',FW);
hL = legend(hP,{'MC','TA upper bound'},'location','south');
set(hL,'interpreter','latex');
set(gca,'ylim',[-82,-10]);
xlabel('Frequency [Hz]','interpreter','latex');
ylabel('Far-end induced voltage [dBV]','interpreter','latex');
set(gca,'position',[0.2,0.2,0.7,0.55]);

