%% ex_TMsim_matrix_inv.m
%
% Reproduces the TM and MC results of the illustative example on the matrix
% inversion (see Fig. 2) in [2] Sec. VII.B
%
% [2] R. Trinchero, P. Manfredi, I. S. Stievano "TMsim: an algorithmic tool
% for the parametric and worst-case simulation of systems with
% uncertainties," submitted to Hindawi, Mathematical Problems in
% Engineering.
% 
% This example script is part of the tool TMsim (TMsim.zip)
% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

% cleaning
clear all
clc
close all

% defining matrix entries
a0 = 1;
av = 0.3;

b0 = 3;
bv = 1;

c0 = pi;
cv = pi/4;
 
d0 = pi;
dv = pi/4;

%% ========================================================================
% Monte Carlo of a bounded Toeplitz matrix
% =========================================================================
Nruns = 20;
xdata = linspace(-1,1,Nruns);

MC_MAX = ones(3)*(-1e9);
MC_min = ones(3)*(1e9);

for ii = 1:length(xdata)

    anl = exp(a0+av*xdata(ii));
    
    bnl = log(b0+bv*xdata(ii));
    
    cnl = sin(c0+cv*xdata(ii));
        
    M = [anl,bnl,cnl;
         bnl,anl,bnl;
         cnl,bnl,anl];
     
    invM(:,:,ii) = M\eye(3);
    
    for jj=1:3
        for kk=1:3
            
            if invM(jj,kk,ii)>MC_MAX(jj,kk)
                MC_MAX(jj,kk) = invM(jj,kk,ii);
            end
            
            if invM(jj,kk,ii)<MC_min(jj,kk)
                MC_min(jj,kk) = invM(jj,kk,ii);
            end
        end
    end
    
end

figure(1)
plot(xdata,squeeze(invM(1,1,:)),'b')
 
%% =========================================================================
% Taylor model 
% =========================================================================

%defining expansion order
N_order = 6;

% inizializing TMsim
initialize_TMsim(N_order,1);

% defining the TM variables
x_TM = TaylorModel(0,1,1);

a_TM = exp(a0+av*x_TM);
b_TM = log(b0+bv*x_TM);
c_TM = sin(c0+cv*x_TM);

M_TM = [a_TM,b_TM,c_TM;
        b_TM,a_TM,b_TM;
        c_TM,b_TM,a_TM];

% inverting the TM matrix
invM_TM = inv(M_TM);

% evaluating the TM over the domain
for ii=1:length(xdata)
    Pval(ii) = evalpoly(invM_TM(1,1).P,xdata(ii));    
end

figure(1)
hold on;
plot(xdata,Pval,'r--')
plot(xdata,Pval+invM_TM(1,1).I(1),'g--')
plot(xdata,Pval+invM_TM(1,1).I(2),'g--')

legend('MC','invM(1,1).P(x)','TM bounds')

TM_min = zeros(3);
TM_MAX = zeros(3);

for ii =1:3
    for jj =1:3
        TM_min(ii,jj) =  invM_TM(ii,jj).B(1)+invM_TM(ii,jj).I(1);
        TM_MAX(ii,jj) =  invM_TM(ii,jj).B(2)+invM_TM(ii,jj).I(2);
    end
end


% visualize the min and max matrix invM computed both with MC and TM
display(MC_min)
display(TM_min)

display(MC_MAX)
display(TM_MAX)

 
