%% ex_TMsim_quartercar_model.m
%
% Reproduces the TM and MC results of application example "Quarter-car
% Model" (see Fig. 2) in [2] Sec. VIII.A
%
% [2] R. Trinchero, P. Manfredi, I. S. Stievano "TMsim: an algorithmic tool
% for the parametric and worst-case simulation of systems with
% uncertainties," submitted to Hindawi, Mathematical Problems in
% Engineering.
% 
% This example script is part of the tool TMsim (TMsim.zip)
% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


% cleaning
clear all;
close all;
clc

% Frequency data
Nf = 30;

f.min = 0.2;
f.max = 100;
freq =  logspace(log10(f.min),log10(f.max),Nf);
w = freq*2*pi;

% System parameters

ms = 332.5000; %front quarter

mu = 38;    %[kg] front unsprung mass of quarter car


K_nom = 22000;     %[N/m] front shock absorber stiffness
C_nom = 6000;      %[Ns/m] front shock absorber damping (realistic value)
P_nom = 2e+05;         %[N/m] front tire stiffness (realistic value)
C_t = 0;

%% =========================================================================
% Monte Carlo (uncorrelated variation, 3 independent parameters)
% =========================================================================
tic;

Nruns = 1000;

% relative variation
delta = 0.2;

progressbar(0);

%random variables
x1 = 2*rand(Nruns,1)-1;
x2 = 2*rand(Nruns,1)-1;
x3 = 2*rand(Nruns,1)-1;

for ii = 1:Nruns
        
    %random parameters
    C_MC = C_nom*(1+delta*x1(ii));
    K_MC = K_nom*(1+delta*x2(ii));
    P_MC = P_nom*(1+delta*x3(ii));
    
    % Mass Matrix  
    Mm = [ms 0
         0  mu];

    % Damping Matrix
    Cm = [C_MC, -C_MC;
         -C_MC, C_MC];

    % Stiffness Matrix
    Km = [K_MC,  -K_MC
         -K_MC, K_MC+P_MC];

    % External Forces Vector
    Fm = [0,0;
         P_MC,C_t];

    % State Matrix
    A = [-inv(Mm)*Cm       -inv(Mm)*Km;
         eye(length(Mm))  zeros(length(Mm))];

    % Input Gain Matrix
    B = [inv(Mm)*Fm;
         zeros(2,2)];   
    
    for kk = 1:length(w)
        X_MC = inv(1j*w(kk)*eye(length(A))-A)*B*[1;1];
        Ysel3(kk,ii) = X_MC(3);        
    end
    
    progressbar(ii/Nruns);

end

MC_time=toc

figure(1)
semilogx(w./(2*pi),real(Ysel3),'k-','color',[0.8,0.8,0.8])
grid on
xlabel('Frequency [Hz]')

figure(2)
semilogx(w./(2*pi),imag(Ysel3),'k-','color',[0.8,0.8,0.8])
grid on
xlabel('Frequency [Hz]')


figure(3)
semilogx(w./(2*pi),20*log10(abs(Ysel3)),'k-','color',[0.8,0.8,0.8])
grid on
xlabel('Frequency [Hz]')


%% =========================================================================
% Taylor model (order 5)
% =========================================================================

tic;

N_order = 5; % expansion order

%initializing
initialize_TMsim(5,3);

% defining the TM variables
C_TM = TaylorModel(C_nom*(1-delta),C_nom*(1+delta),1,'bound');
K_TM = TaylorModel(K_nom*(1-delta),K_nom*(1+delta),2,'bound');
P_TM = TaylorModel(P_nom*(1-delta),P_nom*(1+delta),3,'bound');


one_TM = TaylorModel(1,0,1);
zero_TM = TaylorModel(0,0,1);

eye_TM =[one_TM,zero_TM;
         zero_TM, one_TM];
     
zeros_TM =[zero_TM,zero_TM;
         zero_TM, zero_TM];    
     
dim_sys = 4;
    
% Mass Matrix  
Mm_TM = [ms 0
        0  mu]*one_TM;

% Damping Matrix
Cm_TM = [C_TM,  -1*C_TM;
         -1*C_TM, C_TM];

% Stiffness Matrix
Km_TM = [K_TM  -1*K_TM
        -1*K_TM K_TM+P_TM];

% External Forces Vector
Fm_TM = [zero_TM,zero_TM;
         P_TM,C_t*one_TM];

% State Matrix
A_TM = [-1*inv(Mm_TM)*Cm_TM,-1*inv(Mm_TM)*Km_TM;
     eye_TM,  zeros_TM];

% Input Gain Matrix
B_TM = [inv(Mm_TM)*Fm_TM;
        zeros_TM];

progressbar(0);

    
for jj = 1:length(w)
    
    % solving the system
    X_TM = inv(1j*w(jj)*eye(dim_sys)-1*A_TM)*B_TM*[1;1];
    Ysel3_TM = X_TM(3);

    mag3_TM(:,jj) = TM_abs_bound(Ysel3_TM);
    real3_TM(:,jj) = real(Ysel3_TM.B+Ysel3_TM.I);
    imag3_TM(:,jj) = imag(Ysel3_TM.B+Ysel3_TM.I);
    
    progressbar(jj/Nf);

end

mag3_TM_max = mag3_TM(2,:);
mag3_TM_min = mag3_TM(1,:);


real3_TM_max = real3_TM(2,:);
real3_TM_min = real3_TM(1,:);

imag3_TM_max = imag3_TM(2,:);
imag3_TM_min = imag3_TM(1,:);

TM_time=toc

figure(1)
hold on
semilogx(w./(2*pi),real3_TM(1,:),'-m')
hold on
semilogx(w./(2*pi),real3_TM(2,:),'-m')
xlim([f.min f.max])
grid on
xlabel('Frequency [Hz]')

figure(2)
hold on
semilogx(w./(2*pi),imag3_TM(1,:),'-m')
hold on
semilogx(w./(2*pi),imag3_TM(2,:),'-m')
xlim([f.min f.max])
grid on
xlabel('Frequency [Hz]')


figure(3)
hold on
semilogx(w./(2*pi),mag3_TM(1,:),'-m')
hold on
semilogx(w./(2*pi),mag3_TM(2,:),'-m')
xlim([f.min f.max])
grid on
xlabel('Frequency [Hz]')

