%% ex_TMsim_stripline.m
%
% Reproduces the TM and MC results of stripline example (see Fig. 3) in [3]
% Sec.IV.
%
% [3] P. Manfredi, R. Trinchero, F. G. Canavero and I. S. Stievano,
% "Application of Taylor models to the worst-case analysis of stripline
% interconnects," 2016 IEEE 20th Workshop on Signal and Power Integrity
% (SPI), Turin, 2016, pp. 1-4.
%
%
% This example script is part of the tool TMsim (TMsim.zip)
% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


% cleaning
clear all
clc
close all

% plot parameters
FS=14;
FN='Times';
FW='normal';
LW=1;

GRAY = [0.8,0.8,0.8];


% physical constants
mu0 = 4*pi*1e-7;
eps0 = 8.854e-12;
c0 = 1/sqrt(mu0*eps0);

% line data
L = [4.230891e-007 9.332023e-008;
        9.332023e-008 4.230891e-007];
C0 = [2.764310e-011 -6.097203e-012;
        -6.097203e-012 2.764310e-011];
er = 4;
len = 0.1;

% load data
lin.Vs = [1 0]';
lin.Zs = diag([50,50]);

lin.Il = [0,0]';
Cl = 1e-12;

% variability
var = 0.05;

% frequency sweep data
Nf = 100;
%Nf=10;
freq = linspace(1,2e9,Nf);

%Nf=3;
%freq = [7e8,7.457e8,8e8];

% MC analysis
Nruns = 500;
yruns = unifrnd(-1,1,Nruns,2);

% =========================================================================
% MATLAB simulation
% =========================================================================

VL = zeros(Nf,Nruns,2);
progressbar(0)
for J = 1:Nruns
    er_sample = er*(1+var*yruns(J,1));
    Cl_sample = Cl*(1+var*yruns(J,2));
    
    vf = c0/sqrt(er_sample);
    Zc = vf*L;
    Yc = sqrt(er_sample)*c0*C0;
    
    for I = 1:Nf
        w = 2*pi*freq(I);
        beta = w/vf;
        
        lin.Yl = 1j*w*diag([Cl_sample,Cl_sample]);
                        
        PHI_11 = cos(beta*len)*eye(2);
        PHI_12 = -1j*Zc*sin(beta*len);
        PHI_21 = -1j*Yc*sin(beta*len);
        PHI_22 = cos(beta*len)*eye(2);
        
        % Current at near-end side of the line
        A = PHI_21*lin.Zs + lin.Yl*PHI_12 - PHI_22 - lin.Yl*PHI_11*lin.Zs;
        B = (PHI_21 - lin.Yl*PHI_11)*lin.Vs;

        I_0 = inv(A)*B;

        % Voltage at near-end side of the line
        V_0 = lin.Vs - lin.Zs*I_0;

        % Voltage at far.end side of the line
        VL(I,J,:) = PHI_11*lin.Vs + (PHI_12 - PHI_11*lin.Zs)*I_0;
        
        progressbar(J/Nruns);
    end
end

figure(1)
subplot(2,1,1)
htmp = plot(freq/1e9,20*log10(abs(squeeze(VL(:,:,1)))),'color',GRAY,'linewidth',2); hold on;
hP1(1) = htmp(1);

subplot(2,1,2)
htmp = plot(freq/1e9,20*log10(abs(squeeze(VL(:,:,2)))),'color',GRAY,'linewidth',2); hold on;
hP2(1) = htmp(1);



% =========================================================================
% Taylor models order
% =========================================================================

% Expansion order
N_order = 3; 

% initialize TMsim
initialize_TMsim(N_order,2); 


% parameters in terms of central value + deviation
sqrt_M = sqrt(er*(1+var));
sqrt_m = sqrt(er*(1-var));

sqrt_er_0 = (sqrt_M+sqrt_m)/2;
Delta_sqrt_1 = (sqrt_M-sqrt_m)/2;

C0_l = Cl;
DeltaC_l = Cl*var;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Taylor model of R1 
sqrt_er_T = TaylorModel(sqrt_er_0,Delta_sqrt_1,1);

% Taylor model of C1 
CTl = TaylorModel(C0_l,DeltaC_l,2);

% Taylor model of 1
OneT = TaylorModel(1,0,1);

% Taylor model of -1
OnemT = TaylorModel(-1,0,1);

% Taylor model of 0
ZeroT = TaylorModel(0,0,1);

% diagonal matrix with the loads
for K=1:2
    for I=1:2
        CL_TM(K,I) = ZeroT;
    end
end

CL_TM(1,1) = CTl;
CL_TM(2,2) = CTl;
    
vTf = c0*inv(sqrt_er_T);
ZTc = vTf*L;
YTc = sqrt_er_T*c0*C0;
   
tic
progressbar(0)

for I = 1:Nf
    w = 2*pi*freq(I);
    betaT = w*inv(vTf);

    YTL = 1j*w*CL_TM;
    
    ZTL = Sherman_inv_matrix(YTL);

    PHI_11 = TM_cos(betaT*len)*eye(2);
    PHI_12 = -1j*ZTc*TM_sin(betaT*len);
    PHI_21 = -1j*YTc*TM_sin(betaT*len);
    PHI_22 = TM_cos(betaT*len)*eye(2);
    
   % Current at near-end side of the line
    A = PHI_21*lin.Zs + YTL*PHI_12 - PHI_22 - YTL*PHI_11*lin.Zs;
    B = (PHI_21 - YTL*PHI_11)*lin.Vs;
    
    IT_0 = Sherman_inv_matrix(A)*B;
    
    % Voltage at near-end side of the line
    VT_0 = lin.Vs - lin.Zs*IT_0;

    % Voltage at far.end side of the line
    VT_L(:,I) = PHI_11*lin.Vs + (PHI_12 - PHI_11*lin.Zs)*IT_0;

    % Current at far-end side of the line
    IT_L = YTL*VT_L;
    

    T_Mag_VL1(I,:) = TM_abs_bound(VT_L(1,I));
    T_Mag_VL2(I,:) = TM_abs_bound(VT_L(2,I));

    
    progressbar(I/Nf);
end

%%
figure(1)
subplot(2,1,1)
htmp = semilogx(freq/1e9,T_Mag_VL1,'b-','linewidth',2); hold on; 
hP1(2) = htmp(1);
set(gca,'FontName',FN,'FontSize',FS,'Linewidth',LW,'FontWeight',FW);
hL = legend(hP1,'SPICE (MC)','Taylor Model','location','NorthEast');
set(hL,'interpreter','latex');
set(gca,'ylim',[-2.5,2.5]);
ylabel('Far-end voltage [dBV]','interpreter','latex');


subplot(2,1,2)
semilogx(freq/1e9,T_Mag_VL2,'b-','linewidth',2); hold on;
set(gca,'FontName',FN,'FontSize',FS,'Linewidth',LW,'FontWeight',FW);
set(gca,'ylim',[-60,-10]);
xlabel('Frequency [GHz]','interpreter','latex');
ylabel('Far-end crosstalk [dBV]','interpreter','latex');
