function OUT = Sherman_Unirank(A,B,Col)

% OUT = Sherman_Unirank(A,B,Col) calculates the Sherman-Morrison formula:
% OUT = (A+B)^-1 = A^-1 - 1/(1+g) * A^-1 * B * A^-1 where A and B are Taylor
% model matrices and rank(B)=1. Col is the # of non-zero column of H

% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


%**************************************************************************
%Part I
Ainv = A;     % This is the first part of SM formula: A^(-1) = tmp.

%**************************************************************************
% Part II : Matrix multiplication
% First compute A^(-1) * B, and denote it as AB
B   = B(:,Col);        % Take the non-zero column of H as a N by 1 vector B  
AB  = TM_prod_matrix(Ainv,B);     % Calculate A^(-1) * B (matrix * vector)

% Then compute A^(-1) * B * A^(-1). Denote it as ABA
% First take out a single row of A^-1 as a new vector : Vec_tmp
Vec_tmp = A(Col,:); 
% Then calculate product of two vectors:
% where AB is a N by 1 vector, and the new Vec_tmp is a 1 by N vector
ABA     = TM_prod_matrix(AB,Vec_tmp);      % A^(-1) * B * A^(-1)


% Compute the trace: g = trace( A^(-1) * B ). Denote it as Tr
Tr  = AB(Col,1);       % Trace

Den = TM_sum(1,Tr);    % The Denominator : (1 + g)

COE = TM_inv_scalar(Den);   % The Coefficient of SM formula : 1 / (1+g)


% Part II
% Denote M as the second part of SM formula: M = 1/(1+g) * A^-1 * B * A^-1.
M   = TM_prod_matrix(COE,ABA);    % TM product of a matrix and a scalar


%**************************************************************************
% Output = Part I - Part II
OUT = TM_dif_matrix(Ainv,M);