function Tout_m = Sherman_inv_matrix(T1_m)

% Tout_m = Sherman_inv_matrix(T1_m) inverts a Taylor model matrix T1_m via
% the Sherman-Morrison formula

% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


col = size(T1_m,2);  % Matrix dimension

A = zeros(col);
B = T1_m;

% Zero TM element
Tzero = TaylorModel(0,0,1);

% Split the T1_m as sum of two matrices: 
% A constant matrix A plus a variation matrix B
for I = 1:col
    for J = 1:col
        A(I,J) = T1_m(I,J).P(1);                 % Constant matrix
        B(I,J) = TM_sum(-A(I,J),T1_m(I,J));    % High order terms

        % null matrix
        Tzero_m(I,J) = Tzero;
    end
end
       
A0 = inv(A);
A0 = TM_sum_matrix(Tzero_m,A0);

%**************************************************************************
Tout_m = A0;

% Sherman-Morrison formula
for JJ = 1:col
    
   H       = Tzero_m;
   H(:,JJ) = B(:,JJ);      % H is a Uni-rank matrix obtained from B
   
   Tout_m  = Sherman_Unirank(Tout_m,H,JJ); % Sherman-Morrison subfunction
   
end
