function Tout = TM_cos(T1)

% Tout = TM_cos(T1) calculates the Taylor model representation of the
% cosine function applied to the Taylor model T1

% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


global N_order;   % set order of the polynomial expansion


Tout = TaylorModel;
T1_powk = TaylorModel;

% center of the expansion
cf = T1.P(1);
    
Ttmp = T1-cf; 

% constant Taylor model = 1
T1_powk.P = zeros(size(T1.P));
T1_powk.P(1) = 1;
T1_powk.B = [1,1];
T1_powk.I = [0,0];

% new TM variable Tout (order zero of Taylor expansion)
Tout.P = zeros(size(T1.P));
Tout.P(1) = cos(cf);
Tout.B = [cos(cf),cos(cf)];
Tout.I = [0,0];
    
% Compute the Taylor expansion for cos for order N
for index=1:N_order
    
    T1_powk = T1_powk*Ttmp;
    
    % Taylor expansion cos 
    if (mod(index,2)) %odd
        tmp = (-1)^((index-1)/2+1)/factorial(index)*sin(cf)*T1_powk;
    else
        %even
        tmp = (-1)^(index/2)/factorial(index)*cos(cf)*T1_powk;
    end    
    Tout= TM_sum(Tout,tmp);     
end


%Remainder

% computing the max and minimun values of the cos function over the domain
% defined by the bounds of T1

x_min = T1.B(1)+T1.I(1);
x_max = T1.B(2)+T1.I(2);

%x_min = cf+Ttmp.B(1)+Ttmp.I(1);
%x_max = cf+Ttmp.B(2)+Ttmp.I(2);
      
% Number of turns
Nturns = floor(abs(x_max-x_min)/(2*pi));

% wraping the domain of the argumnent T1, in radians, to the interval 
% [0,2*pi]
x_min = my_wrapTo2Pi(x_min);
x_max = my_wrapTo2Pi(x_max);
        

T1_powk = T1_powk*Ttmp;

flag = 1;

if (mod(N_order+1,2)) 
    
    % N_order+1 odd   
    if Nturns>=1
        
        % the max and min of the sin is within the domain of T1 
        R1 = (-1)^((N_order+1-1)/2+1)/factorial(N_order+1)*IA_prod([-1,1],T1_powk.B+T1_powk.I);
        R2 = R1;
        flag = 0;
    else
        if and(x_min<=pi/2,x_max>=pi/2)
            
            %the max of the cos is within the domain of T1
            R1 = (-1)^((N_order+1-1)/2+1)/factorial(N_order+1)*IA_prod([min(sin(T1.B+T1.I)),1],T1_powk.B+T1_powk.I);
            R2 = [0,0];
            flag = 0;
        end
        
        if and(x_min<=3*pi/2,x_max>=3*pi/2)
            
            %the min of the sin is within the domain of T1
            R1 = [0,0];
            R2 = (-1)^((N_order+1-1)/2+1)/factorial(N_order+1)*IA_prod([-1,max(sin(T1.B+T1.I))],T1_powk.B+T1_powk.I);
            flag = 0;
        end
        
        if flag
            
            % the min and max of sin over the domain [x_min,x_max] is given by
            % the min/max values of sin([x_min,x_max])           
            R1 = (-1)^((N_order+1-1)/2+1)/factorial(N_order+1)*IA_prod(sin(T1.B+T1.I),T1_powk.B+T1_powk.I);
            R2 = R1;
        end
    end
else
    
    %N_order+1 even
    if Nturns>=1
        
        % the max and min of the cos is within the domain of T1 
        R1 = (-1)^((N_order+1)/2)/factorial(N_order+1)*IA_prod([-1,1],T1_powk.B+T1_powk.I);
        R2 = R1;
        flag = 0;
    else
        if and(x_min<=0,x_max>=0)
            
            %the max of the cos is within the domain of T1
            R1 = (-1)^((N_order+1)/2)/factorial(N_order+1)*IA_prod([min(cos(T1.B+T1.I)),1],T1_powk.B+T1_powk.I);
            R2 = [0,0];
            flag = 0;
        end
        
        if and(x_min<=pi,x_max>=pi)
        
            %the min of the cos is within the bound of T1
            R1 = [0,0];
            R2 = (-1)^((N_order+1)/2)/factorial(N_order+1)*IA_prod([-1,max(cos(T1.B+T1.I))],T1_powk.B+T1_powk.I);
            flag = 0;
        end
        
        if flag 
            
            % the min and max of cos over the domain [x_min,x_max] is given by
            % the min/max values of cos([x_min,x_max])
            R1 = (-1)^((N_order+1)/2)/factorial(N_order+1)*IA_prod(cos(T1.B+T1.I),T1_powk.B+T1_powk.I);
            R2 = R1;
        end
    end

    
end  

% final remainder       
Tout.I(1) = Tout.I(1)+min([R1(1),R1(2),R2(1),R2(2)]);
Tout.I(2) = Tout.I(2)+max([R1(1),R1(2),R2(1),R2(2)]);





