classdef TaylorModel
% defining the class Taylor model

% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


    properties
        P %polynomial part up to the order Norder
        B % bounds of the polynomial 
        I % remainder to account for the tuncation order
    end


    methods
        
%initializing a new Taylor model variable of a bounded parameter [v-dv,v+dv] 
        function out = TaylorModel(a,b,index,interval)                   
            global EXP XRANGE
            if nargin>0
                if nargin<4 || strcmpi(interval,'center')
                    v0 = a;
                    dv = b;
                elseif strcmpi(interval,'bound')
                    v0 = (b+a)/2;
                    dv = (b-a)/2;
                else
                    error('Interval specification not correct')
                end
                coeff_len = size(EXP,2);
                out.P = zeros(1,coeff_len);
                out.P(1) = v0;
                out.P(1+index) = dv;
                out.B = [v0+dv*XRANGE(index,1),v0+dv*XRANGE(index,2)];
                out.I = [0,0];
            end
        end

        
        
% new definition of the standard operations according to the Taylor
% model arithmetic        
        function Tout = plus(T1,T2)
            Tout = TM_sum_matrix(T1,T2);
        end
        
        function Tout = minus(T1,T2)
            Tout = TM_dif_matrix(T1,T2);
        end
        
        function Tout = mtimes(T1,T2)
            Tout = TM_prod_matrix(T1,T2);
        end
        
        function Tout = mpower(T1,k)
            if k==-1
                Tout = inv(T1);
            else
                Tout = TM_pow(T1,k);
            end
        end
        
        function Tout = inv(T1)
            [M1,N1] = size(T1);
            if and((M1==1),(N1==1))
                Tout = TM_inv_scalar(T1);
            else
                Tout = Sherman_inv_matrix(T1);
            end
        end
        
        function Tout = log(T1)
            Tout = TM_log(T1);
        end
        
        function Tout = sin(T1)
            Tout = TM_sin(T1);
        end
        
        function Tout = cos(T1)
            Tout = TM_cos(T1);
        end
        
        function Tout = exp(T1)
            Tout = TM_exp(T1);
        end
        
        function Tout = acosh(T1)
            Tout = TM_acosh(T1);
        end
        
        function Tout = sqrt(T1)
            Tout = TM_sqrt(T1);
        end
        
   end
end